@@ -1,4 +1,4 @@
-/* $Id: gweather-applet.c,v 1.98 2005/08/25 10:02:18 rodrigo Exp $ */
+/* $Id: gweather-applet.c,v 1.102 2006/01/12 05:48:25 philipl Exp $ */
 
 /*
  *  Papadimitriou Spiros <spapadim+@cs.cmu.edu>
@@ -27,7 +27,6 @@
 #include <libnotify/notify.h>
 #endif
 
-#include "weather.h"
 #include "gweather.h"
 #include "gweather-about.h"
 #include "gweather-pref.h"
@@ -134,48 +133,6 @@ static void size_allocate_cb(PanelApplet *w, GtkAllocation *allocation, gpointer
     return;
 }
 
-static void change_background_cb (
-				PanelApplet *a, 
-				PanelAppletBackgroundType type,
-				GdkColor *color, GdkPixmap *pixmap, 
-				gpointer data)
-{
-	/* taken from the TrashApplet */
-	GWeatherApplet *gw_applet = (GWeatherApplet *) data;
-	GtkRcStyle *rc_style;
-	GtkStyle *style;
-
-	/* reset style */
-	gtk_widget_set_style (GTK_WIDGET (gw_applet->applet), NULL);
-	rc_style = gtk_rc_style_new ();
-	gtk_widget_modify_style (GTK_WIDGET (gw_applet->applet), rc_style);
-	g_object_unref (rc_style);
-
-	switch (type) {
-		case PANEL_COLOR_BACKGROUND:
-			gtk_widget_modify_bg (GTK_WIDGET (gw_applet->applet),
-					GTK_STATE_NORMAL, color);
-			break;
-
-		case PANEL_PIXMAP_BACKGROUND:
-			style = gtk_style_copy (
-					GTK_WIDGET (gw_applet->applet)->style);
-			if (style->bg_pixmap[GTK_STATE_NORMAL])
-				g_object_unref
-					(style->bg_pixmap[GTK_STATE_NORMAL]);
-			style->bg_pixmap[GTK_STATE_NORMAL] = g_object_ref
-				(pixmap);
-			gtk_widget_set_style (GTK_WIDGET (gw_applet->applet),
-					style);
-			break;
-
-		case PANEL_NO_BACKGROUND:
-		default:
-			break;
-	}
-}
-
-
 static gboolean clicked_cb (GtkWidget *widget, GdkEventButton *ev, gpointer data)
 {
     GWeatherApplet *gw_applet = data;
@@ -219,7 +176,6 @@ key_press_cb (GtkWidget *widget, GdkEventKey *event, GWeatherApplet *gw_applet)
 	case GDK_KP_Space:
 		gweather_dialog_open(gw_applet);
 		return TRUE;
-		break;
 	default:
 		break;
 	}
@@ -308,30 +264,18 @@ applet_destroy (GtkWidget *widget, GWeatherApplet *gw_applet)
        gw_applet->timeout_tag = 0;
     }
 	
-    if (gw_applet->gweather_info->metar_handle) {
-       gnome_vfs_async_cancel(gw_applet->gweather_info->metar_handle);
-       gw_applet->gweather_info->metar_handle = NULL;
+    if (gw_applet->suncalc_timeout_tag > 0) {
+       gtk_timeout_remove(gw_applet->suncalc_timeout_tag);
+       gw_applet->suncalc_timeout_tag = 0;
+    }
+	
+    if (gw_applet->gconf) {
+       gweather_gconf_free (gw_applet->gconf);
     }
 
-    if (gw_applet->gweather_info->iwin_handle) {
-       gnome_vfs_async_cancel(gw_applet->gweather_info->iwin_handle);
-       gw_applet->gweather_info->iwin_handle = NULL;
-    }
+    weather_info_abort (gw_applet->gweather_info);
 
-    if (gw_applet->gweather_info->wx_handle) {
-       gnome_vfs_async_cancel(gw_applet->gweather_info->wx_handle);
-       gw_applet->gweather_info->wx_handle = NULL;
-    }
-
-    if (gw_applet->gweather_info->met_handle) {
-       gnome_vfs_async_cancel(gw_applet->gweather_info->met_handle);
-       gw_applet->gweather_info->met_handle = NULL;
-    }
-
-    if (gw_applet->gweather_info->bom_handle) {
-       gnome_vfs_async_cancel(gw_applet->gweather_info->bom_handle);
-       gw_applet->gweather_info->bom_handle = NULL;
-    }
+    gtk_main_quit ();
 }
 
 void gweather_applet_create (GWeatherApplet *gw_applet)
@@ -353,6 +297,9 @@ void gweather_applet_create (GWeatherApplet *gw_applet)
 	gw_applet->gweather_pref.distance_unit = DISTANCE_UNIT_INVALID;
     
     panel_applet_set_flags (gw_applet->applet, PANEL_APPLET_EXPAND_MINOR);
+
+    panel_applet_set_background_widget(gw_applet->applet,
+                                       GTK_WIDGET(gw_applet->applet));
     
     icon_info = gtk_icon_theme_lookup_icon (gtk_icon_theme_get_default (), "stock_weather-storm", 48, 0);
     if (icon_info) {
@@ -367,8 +314,6 @@ void gweather_applet_create (GWeatherApplet *gw_applet)
                        G_CALLBACK(change_orient_cb), gw_applet);
     g_signal_connect (G_OBJECT(gw_applet->applet), "size_allocate",
                        G_CALLBACK(size_allocate_cb), gw_applet);
-    g_signal_connect (G_OBJECT(gw_applet->applet), "change_background",
-		       G_CALLBACK(change_background_cb), gw_applet);
     g_signal_connect (G_OBJECT(gw_applet->applet), "destroy", 
                        G_CALLBACK (applet_destroy), gw_applet);
     g_signal_connect (GTK_OBJECT(gw_applet->applet), "button_press_event",
@@ -421,31 +366,37 @@ gint timeout_cb (gpointer data)
     return 0;  /* Do not repeat timeout (will be re-set by gweather_update) */
 }
 
-void update_finish (WeatherInfo *info)
+static void
+update_finish (WeatherInfo *info, gpointer data)
 {
     static int gw_fault_counter = 0;
 #ifdef HAVE_LIBNOTIFY
-    static NotifyIcon *icon = NULL;
-    NotifyHints *hints;
-    char *notification_message, *notification_detail;
+    char *message, *detail;
     GdkPixbuf *pixbuf = NULL;
     GConfClient *conf;
-    int x, y;
-    GtkRequisition size;
 #endif
     char *s;
-    GWeatherApplet *gw_applet = info->applet;
-   
+    GWeatherApplet *gw_applet = (GWeatherApplet *)data;
+    gint nxtSunEvent;
+
     /* Update timer */
     if (gw_applet->timeout_tag > 0)
         gtk_timeout_remove(gw_applet->timeout_tag);
     if (gw_applet->gweather_pref.update_enabled)
-        gw_applet->timeout_tag =  
-        	gtk_timeout_add (
-			gw_applet->gweather_pref.update_interval * 1000,
+    {
+	gw_applet->timeout_tag =
+		gtk_timeout_add (
+                       gw_applet->gweather_pref.update_interval * 1000,
                         timeout_cb, gw_applet);
- 
-    if ((TRUE == info->valid) ||
+
+        nxtSunEvent = weather_info_next_sun_event(gw_applet->gweather_info);
+        if (nxtSunEvent >= 0)
+            gw_applet->suncalc_timeout_tag =
+                        gtk_timeout_add (nxtSunEvent * 1000,
+                                suncalc_timeout_cb, gw_applet);
+    }
+
+    if ((TRUE == weather_info_is_valid (info)) ||
 	     (gw_fault_counter >= MAX_CONSECUTIVE_FAULTS))
     {
 	    gw_fault_counter = 0;
@@ -462,10 +413,10 @@ void update_finish (WeatherInfo *info)
 	    gtk_tooltips_set_tip (gw_applet->tooltips, GTK_WIDGET (
 				    gw_applet->applet), s, NULL);
 	    g_free (s);
-	    
+
 	    /* Update dialog -- if one is present */
 	    gweather_dialog_update(gw_applet);
-	    
+
 #ifdef HAVE_LIBNOTIFY
             conf = gconf_client_get_default ();
             if (!gconf_client_get_bool (conf, "/apps/panel/applets/weather/use_libnotify", NULL))
@@ -479,60 +430,29 @@ void update_finish (WeatherInfo *info)
             if (!notify_is_initted ())
                 if (!notify_init (_("Weather Forecast")))
                     return;
-            
-            /* Get the icon for the notification message */
-            if (icon) {
-                notify_icon_destroy (icon);
-                icon = NULL;
-            }
-            weather_info_get_pixbuf (gw_applet->gweather_info, &pixbuf);
-            if (pixbuf) {
-                gchar *tmp;
-                GError *error = NULL;
-                
-                tmp = g_strdup_printf ("%s/.gnome2/gweather-icon.png", g_get_home_dir ());
-                if (gdk_pixbuf_save (pixbuf, tmp, "png", &error, NULL))
-                    icon = notify_icon_new_from_uri (tmp);
-                else
-                    g_warning ("Unable to save weather icon: %s\n", error->message);
-                
-                if (error != NULL)
-                    g_error_free (error);
-                
-                g_free (tmp);
-            }
-            
-	    gdk_window_get_origin (GTK_WIDGET (gw_applet->applet)->window, &x, &y);
-	    gtk_widget_size_request (GTK_WIDGET (gw_applet->applet), &size);
-	    x += size.width / 2;
-	    y += size.height;
 
-	    hints = notify_hints_new ();
-	    notify_hints_set_int (hints, "x", x);
-	    notify_hints_set_int (hints, "y", y);
-	    
             /* Show notification */
-            notification_message = g_strdup_printf ("%s: %s",
-                                                    weather_info_get_location (info),
-                                                    weather_info_get_sky (info));
-            notification_detail = g_strdup_printf (_("City: %s\nSky: %s\nTemperature: %s"),
-            		                           weather_info_get_location (info),
-            		                           weather_info_get_sky (info),
-            		                           weather_info_get_temp_summary (info));
+            message = g_strdup_printf ("%s: %s",
+				       weather_info_get_location_name (info),
+				       weather_info_get_sky (info));
+            detail = g_strdup_printf (_("City: %s\nSky: %s\nTemperature: %s"),
+				      weather_info_get_location_name (info),
+				      weather_info_get_sky (info),
+				      weather_info_get_temp_summary (info));
             
-            if (!notify_send_notification (NULL, "transfer",
-                                           NOTIFY_URGENCY_LOW,
-	                                   notification_message,
-	                                   notification_detail,	/* body text */
-	                                   icon,		/* icon */
-	                                   TRUE, 0,		/* expiry, server default */
-	                                   hints,		/* hints */
-	                                   NULL,		/* no user_data */
-	                                   0))			/* no actions */
+	    NotifyNotification *n = notify_notification_new(message, detail,
+							    GTK_STOCK_INFO, 
+							    gw_applet->container);
+
+	    /* FIXME: emable this makes notify-daemon crash */
+/*             weather_info_get_pixbuf (gw_applet->gweather_info, &pixbuf); */
+/* 	    if (pixbuf) */
+/* 	       notify_notification_set_icon_data_from_pixbuf (n, pixbuf); */
+	    if (!notify_notification_show (n, NULL))
 	        g_warning ("Could not send notification to daemon\n");
 
-	    g_free (notification_message);
-	    g_free (notification_detail);
+	    g_free (message);
+	    g_free (detail);
 #endif
     }
     else
@@ -544,9 +464,17 @@ void update_finish (WeatherInfo *info)
     }
 }
 
+gint suncalc_timeout_cb (gpointer data)
+{
+    WeatherInfo *info = ((GWeatherApplet *)data)->gweather_info;
+    update_finish(info, data);
+    return 0;  /* Do not repeat timeout (will be re-set by update_finish) */
+}
+
+
 void gweather_update (GWeatherApplet *gw_applet)
 {
-    gboolean update_success;
+    WeatherPrefs prefs;
 
     weather_info_get_pixbuf_mini(gw_applet->gweather_info, 
     				 &(gw_applet->applet_pixbuf));
@@ -557,23 +485,30 @@ void gweather_update (GWeatherApplet *gw_applet)
     			 _("Updating..."), NULL);
 
     /* Set preferred forecast type */
-    weather_forecast_set(gw_applet->gweather_pref.detailed ? 
-    			 	FORECAST_ZONE : FORECAST_STATE);
+    prefs.type = gw_applet->gweather_pref.detailed ? FORECAST_ZONE : FORECAST_STATE;
 
     /* Set radar map retrieval option */
-    weather_radar_set(gw_applet->gweather_pref.radar_enabled);
+    prefs.radar = gw_applet->gweather_pref.radar_enabled;
+    prefs.radar_custom_url = (gw_applet->gweather_pref.use_custom_radar_url &&
+    				gw_applet->gweather_pref.radar) ?
+				gw_applet->gweather_pref.radar : NULL;
+
+    /* Set the units */
+    prefs.temperature_unit = gw_applet->gweather_pref.temperature_unit;
+    prefs.speed_unit = gw_applet->gweather_pref.speed_unit;
+    prefs.pressure_unit = gw_applet->gweather_pref.pressure_unit;
+    prefs.distance_unit = gw_applet->gweather_pref.distance_unit;
 
     /* Update current conditions */
     if (gw_applet->gweather_info && 
-    	weather_location_equal(gw_applet->gweather_info->location, 
+    	weather_location_equal(weather_info_get_location(gw_applet->gweather_info),
     			       gw_applet->gweather_pref.location)) {
-        update_success = weather_info_update(gw_applet, 
-        				      gw_applet->gweather_info, 
-        				      update_finish);
+	weather_info_update(gw_applet->gweather_info, &prefs,
+			    update_finish, gw_applet);
     } else {
         weather_info_free(gw_applet->gweather_info);
-        gw_applet->gweather_info = NULL;
-        update_success = weather_info_new((gpointer)gw_applet, 
-        		 gw_applet->gweather_pref.location, update_finish);
+        gw_applet->gweather_info = weather_info_new(gw_applet->gweather_pref.location,
+						    &prefs,
+						    update_finish, gw_applet);
     }
 }
