@@ -24,6 +24,7 @@
 #include "config.h"
 #endif
 
+#include <math.h>
 #include <string.h>
 
 #include <glib-object.h>
@@ -98,19 +99,10 @@ static void	cb_ui_event			(BonoboUIComponent *comp,
 						 gpointer           data);
 static void	cb_theme_change                (GtkIconTheme *icon_theme,
 						gpointer      data);
+static void	cb_stop_scroll_events		(GtkWidget *widget,
+						 GdkEvent  *event);
 
 static PanelAppletClass *parent_class = NULL;
-static struct {
-  gchar *filename;
-  GdkPixbuf *pixbuf;
-} pix[] = {
-  { "stock_volume-mute", NULL },
-  { "stock_volume-0",    NULL },
-  { "stock_volume-min",  NULL },
-  { "stock_volume-med",  NULL },
-  { "stock_volume-max",  NULL },
-  { NULL, NULL }
-};
 
 GType
 gnome_volume_applet_get_type (void)
@@ -143,24 +135,32 @@ gnome_volume_applet_get_type (void)
 static void
 init_pixbufs (GnomeVolumeApplet *applet)
 {
+  static const gchar *pix_filenames[] = {
+    "stock_volume-mute",
+    "stock_volume-0",
+    "stock_volume-min",
+    "stock_volume-med",
+    "stock_volume-max",
+    NULL
+  };
   gint n;
   
-  for (n = 0; pix[n].filename != NULL; n++) {
-    if (pix[n].pixbuf)
-      g_object_unref (pix[n].pixbuf);
+  for (n = 0; pix_filenames[n] != NULL; n++) {
+    if (applet->pix[n])
+      g_object_unref (applet->pix[n]);
     
-    pix[n].pixbuf = gtk_icon_theme_load_icon (
+    applet->pix[n] = gtk_icon_theme_load_icon (
       	    applet->icon_theme,
-      	    pix[n].filename,
+      	    pix_filenames[n],
       	    panel_applet_get_size (&applet->parent),
       	    0,
       	    NULL);
     if (gtk_widget_get_default_direction () == GTK_TEXT_DIR_RTL) {
       GdkPixbuf *temp;
 
-      temp = gdk_pixbuf_flip (pix[n].pixbuf, TRUE);
-      g_object_unref (G_OBJECT (pix[n].pixbuf));
-      pix[n].pixbuf = temp;
+      temp = gdk_pixbuf_flip (applet->pix[n], TRUE);
+      g_object_unref (G_OBJECT (applet->pix[n]));
+      applet->pix[n] = temp;
     }
   }
 }
@@ -201,6 +201,7 @@ gnome_volume_applet_init (GnomeVolumeApplet *applet)
   applet->state = -1;
   applet->prefs = NULL;
   applet->icon_theme = gtk_icon_theme_get_default ();
+  applet->dock = NULL;
 
   /* init pixbufs */
   init_pixbufs (applet);
@@ -220,6 +221,11 @@ gnome_volume_applet_init (GnomeVolumeApplet *applet)
   gtk_tooltips_set_tip (applet->tooltips, GTK_WIDGET (applet),
 			_("Volume Control"), NULL);
 
+  /* prevent scroll events from reaching the tooltip */
+  g_signal_connect (G_OBJECT (applet),
+		    "event-after", G_CALLBACK (cb_stop_scroll_events),
+		    NULL);
+
   /* handle icon theme changes */
   g_signal_connect (G_OBJECT (applet->icon_theme),
 		    "changed", G_CALLBACK (cb_theme_change),
@@ -311,7 +317,6 @@ select_element_and_track (GnomeVolumeApplet *applet,
 			  const char        *active_track_names)
 {
   GstElement *active_element;
-  GstMixerTrack *active_track;
   GList *active_tracks, *l;
 
   applet->elements = elements;
@@ -364,7 +369,6 @@ gnome_volume_applet_setup (GnomeVolumeApplet *applet,
 			   GList *elements)
 {
   GtkObject *adj;
-  gint page;
   BonoboUIComponent *component;
   static const BonoboUIVerb verbs[] = {
     BONOBO_UI_UNSAFE_VERB ("RunMixer", cb_verb),
@@ -449,6 +453,7 @@ static void
 gnome_volume_applet_dispose (GObject *object)
 {
   GnomeVolumeApplet *applet = GNOME_VOLUME_APPLET (object);
+  gint n;
 
   gnome_volume_applet_popdown_dock (applet);
 
@@ -483,6 +488,13 @@ gnome_volume_applet_dispose (GObject *object)
     applet->timeout = 0;
   }
 
+  for (n = 0; n < 5; n++) {
+    if (applet->pix[n] != NULL) {
+      g_object_unref (G_OBJECT (applet->pix[n]));
+      applet->pix[n] = NULL;
+    }
+  }
+
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
@@ -571,6 +583,8 @@ gnome_volume_applet_popup_dock (GnomeVolumeApplet *applet)
   gint x, y;
 
   /* show (before reposition, so size allocation is done) */
+  gnome_volume_applet_get_dock_position (applet, &x, &y);
+  gtk_window_move (GTK_WINDOW (applet->dock), x, y);
   gtk_widget_show_all (GTK_WIDGET (applet->dock));
 
   /* reposition */
@@ -830,6 +844,9 @@ gnome_volume_applet_key (GtkWidget   *widget,
       GtkAdjustment *adj = gtk_range_get_adjustment (applet->dock->scale);
       gdouble volume = adj->value, increment;
 
+      if (event->state != 0)
+        break;
+
       if (event->keyval == GDK_Up || event->keyval == GDK_Down)
         increment = adj->step_increment;
       else
@@ -875,7 +892,7 @@ gnome_volume_applet_orientation	(PanelApplet *_applet,
      *    thinks that the child of the applet (image) is us (dock),
      *    which is not the case.
      */
-    gtk_widget_destroy (GTK_WIDGET (applet->dock));
+    gtk_widget_unparent (GTK_WIDGET (applet->dock));
   }
   dock = gnome_volume_applet_dock_new (IS_PANEL_HORIZONTAL (orientation) ?
       GTK_ORIENTATION_VERTICAL : GTK_ORIENTATION_HORIZONTAL);
@@ -989,7 +1006,6 @@ cb_volume (GtkAdjustment *adj,
 	   gpointer data)
 {
   GnomeVolumeApplet *applet = data;
-  gint *volumes, n;
   gdouble v;
   GList *iter;
 
@@ -1056,9 +1072,9 @@ gnome_volume_applet_refresh (GnomeVolumeApplet *applet,
 
   if (did_change) {
     if (mute) {
-      pixbuf = pix[0].pixbuf;
+      pixbuf = applet->pix[0];
     } else {
-      pixbuf = pix[n].pixbuf;
+      pixbuf = applet->pix[n];
     }
 
     gtk_image_set_from_pixbuf (applet->image, pixbuf);
@@ -1180,7 +1196,6 @@ cb_gconf (GConfClient *client,
       if (active_tracks) {
         GtkObject *adj;
 	GstMixerTrack *first_track;
-        gint page, percent;
 
 	/* copy the newly created track list over to the main list */
 	g_list_free (applet->tracks);
@@ -1319,3 +1334,16 @@ cb_theme_change (GtkIconTheme *icon_theme,
   init_pixbufs (applet);
   gnome_volume_applet_refresh (applet, TRUE);
 }
+
+/*
+ * Block the tooltips event-after handler on scroll events.
+ */
+
+static void
+cb_stop_scroll_events (GtkWidget *widget,
+		       GdkEvent  *event)
+{
+  if (event->type == GDK_SCROLL)
+    g_signal_stop_emission_by_name (widget, "event-after");
+}
+
