@@ -24,13 +24,21 @@
 
 #include <panel-applet.h>
 #include <fcntl.h>
+#ifdef HAVE_PTY_H
 #include <pty.h>
+#endif
 #include <sys/poll.h>
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <libxml/tree.h>
 #include <glade/glade.h>
 
+#ifdef __FreeBSD__
+#include <sys/ioctl.h>
+#include <termios.h>
+#include <libutil.h>
+#endif
+
 #include "modem-applet.h"
 
 #define MODEM_APPLET_GET_PRIVATE(obj)  (G_TYPE_INSTANCE_GET_PRIVATE ((obj), TYPE_MODEM_APPLET, ModemAppletPrivate))
@@ -348,7 +356,7 @@ find_next_element (xmlNodePtr node, const gchar *name)
   return n;
 }
 
-static gchar *
+static guchar *
 element_get_attribute (xmlNodePtr node, const gchar *attribute)
 {
   xmlAttrPtr a;
@@ -367,7 +375,7 @@ element_get_attribute (xmlNodePtr node, const gchar *attribute)
   return NULL;
 }
 
-static gchar *
+static guchar *
 element_get_child_content (xmlNodePtr node, const gchar *tag)
 {
   xmlNodePtr child, n;
@@ -605,6 +613,12 @@ shutdown_backend (ModemApplet *applet, gboolean backend_alive, gboolean already_
       priv->timeout_id = 0;
     }
 
+  if (priv->tooltip_id)
+    {
+      g_source_remove (priv->tooltip_id);
+      priv->tooltip_id = 0;
+    }
+
   if (backend_alive)
     kill (priv->pid, 9);
 
@@ -800,6 +814,9 @@ update_info_callback (ModemApplet *applet, xmlDoc *doc)
     return;
 
   iface = find_dialup_interface_node (get_root_node (doc));
+  if (!iface)
+    return;
+
   get_interface_data (applet, iface);
   update_popup_buttons (applet);
 }
@@ -1023,13 +1040,13 @@ on_modem_applet_about_clicked (BonoboUIComponent *uic,
   };
 */
   gtk_show_about_dialog (NULL,
-			 "name",               _("Modem monitor"),
+			 "name",               _("Modem Monitor"),
 			 "version",            VERSION,
 			 "copyright",          "Copyright \xC2\xA9 2004 Free Software Foundation. Inc.",
 			 "comments",           _("Applet for activating and monitoring a dial-up network connection."),
 			 "authors",            authors,
 			 /* "documenters",        documenters, */
-			 "translator-credits", _("translator_credits"),
+			 "translator-credits", _("translator-credits"),
 			 "logo_icon_name",     "gnome-modem",
 			 NULL);
 }
